@@ -59,10 +59,5 @@ class Fs_radius_vsas_config_class extends Safe_configurable_object_class
         $cfg = $this->get_data_values(false);
         return "Vendor=" . $cfg['vendor-id'] . " ID=" . $cfg['vsa-id'];
     }
-    public function summary($type = 'horizontal' , $long = false){
-        $table_line = parent::summary($type , $long);
-        array_shift($table_line['data']);
-        return $table_line;
-    }
 }
 ?>
