@@ -432,7 +432,7 @@ class Fs_sip_profile_class extends Safe_configurable_object_class
       //check system port
       foreach($ports as $port_item){
           if(!$this->node()->os()->valid_ip_port($port_item['ip'], $port_item['port'], $port_item['type'], 'nsc')){
-              $error_msg_2 = " This port already using by other program now. " ;
+              $error_msg_2 = " This port is already being used by another program. " ;
           }
       }
       if($error_msg == '' && $error_msg_2 == ''){
