@@ -249,7 +249,7 @@ function safe_form_with_submit_button($message, $action, $button_label, $button_
 {
     if (isset($button_value)) $button_name = ' name="' . $button_value . '" ';
     else $button_name = '';
-    if (isset($style)) $style.= ' style="' . $style . '" ';
+    if (isset($style)) $style = ' style="' . $style . '" ';
     else $style = "";
     $extra_fields = "";
     foreach ($hidden_fields as $k => $v) {
@@ -259,11 +259,22 @@ function safe_form_with_submit_button($message, $action, $button_label, $button_
     <form action="' . $action . '" method="' . $method . '" enctype="multipart/form-data">' . $message . $extra_fields . '<input ' . $button_name . $style . ' value="' . $button_label . '" type="submit" class="ui-state-default ui-corner-all">
     </form>';
 }
+function safe_form_action_button($message, $action, $button_label, $button_value = null, $method = 'post', $style = null, $hidden_fields = array())
+{
+    $tmp_button  = safe_form_with_submit_button($message, $action, $button_label, $button_value, $method, $style, $hidden_fields);
+    $tmp_button = str_replace('type="submit"','type="button" onclick="safe_handle_create_button(this);"', $tmp_button);
+    $tmp_button = '<div style="float:left">'.$tmp_button.'</div>';
+    return $tmp_button;
+}
+function safe_form_global_action($message = '', $buttons ='')
+{
+    return '<div class="safe_notif_warn">' . $message . ' <span style=" margin-left: 5px; margin-right: 5px;"> : </span> </div>' . $buttons;
+}
 function safe_form_with_submit_button_confirm($message, $action, $button_label, $button_value = NULL, $method = 'post', $style = NULL, $hidden_fields = array())
 {
     if (isset($button_value)) $button_name = ' name="' . $button_value . '" ';
     else $button_name = '';
-    if (isset($style)) $style.= ' style="' . $style . '" ';
+    if (isset($style)) $style = ' style="' . $style . '" ';
     else $style = "";
     $extra_fields = "";
     foreach ($hidden_fields as $k => $v) {
@@ -277,16 +288,10 @@ function safe_form_with_submit_button_confirm($message, $action, $button_label,
         . '" type="button" onclick="safe_handle_confirm_button(this);" class="ui-state-default ui-corner-all">
     </form>';
 }
-function safe_button_click($name, $value, $url, $id = '', $attributes = NULL)
+function safe_button_click($name, $value, $url, $id = '', $attributes = null)
 {
-    if (!isset($attributes)) {
-        $attributes = array(
-            'enctype' => 'multipart/form-data',
-            'class' => 'ui-state-default ui-corner-all',
-        );
-    }
     $html = '
-    <input name="' . $name . '" value="' . $value . '" type="button" ';
+    <input name="' . $name . '" value="' . $value . '" type="button" class="ui-state-default ui-corner-all" ';
     foreach ($attributes as $attr => $attr_value) {
         $html.= ' ' . $attr . '="' . $attr_value . '"';
     }
