@@ -135,6 +135,7 @@ class Safe_audit_point extends Safe_object_class
         // Fields parameters
         $params = $parameters['param'];
         // Strip value_fields
+        $default_value = $this->_value_fields;
         $this->_value_fields = array();
         if($params){
             foreach($params as $param => $def){
@@ -144,8 +145,25 @@ class Safe_audit_point extends Safe_object_class
                 if(!$this->_value_fields[$param]['label']){
                     $this->_value_fields[$param]['label'] = ucfirst($param);
                 }
+                $help_msg = array();
+                if($default_value[$param]['help']){
+                    $help_msg = to_index_array($default_value[$param]['help']);
+                }
+                if($this->_value_fields[$param]['help']){
+                   $help_msg = array_merge( $help_msg ,to_index_array($this->_value_fields[$param]['help']));
+                }
+                if($help_msg){
+                    $this->_value_fields[$param]['help'] = $help_msg;
+                }
             }
         }
+        // Fields help
+        $help = $parameters['help'];
+        if($help){
+            $help = to_index_array($help);
+            $this->set_documentation($help);
+        }
+        
     }
     protected function _field_layout($configurable)
     {
@@ -181,6 +199,8 @@ class Safe_audit_point extends Safe_object_class
             $this->_action,
             //$configurable->enable_disable_enum());
             $action);
+        $help = array('Enable/Disable ' . $this->description() . ' audit point.');
+        $configurable->set_field_help($this->name().'/action', $help);
     }
     static public function severity_enum()
     {
@@ -203,6 +223,7 @@ class Safe_audit_point extends Safe_object_class
             'dropdown',
             $this->_level,
             $this->_level_enum());
+        $configurable->set_field_help($this->name().'/level','Select the severity level.');
     }
 
     protected function _field_value_def($configurable)
@@ -244,6 +265,19 @@ class Safe_audit_point extends Safe_object_class
         $this->_field_action_def($configurable);
         $this->_field_level_def($configurable);
         $this->_field_value_def($configurable);
+        
+        //generate help messages before composite_layout, composite_layout will process the help msg
+        $fields = array_merge(array('action','level'), array_keys($this->_value_fields));
+        $all_help_msg =  array();
+        foreach($fields as $field_name){
+            $field_name = $this->name() . '/' . $field_name;
+            $tmp_help_msg = $configurable->get_field_help($field_name);
+            if($tmp_help_msg){
+                $all_help_msg[][$configurable->get_data_label($field_name)] = $tmp_help_msg;
+                $configurable->set_field_help($field_name, array());
+            }
+        }
+        
         $this->_field_layout($configurable);
         // Create fields in group
         $row_fields = array($this->name().'/level');
@@ -257,9 +291,14 @@ class Safe_audit_point extends Safe_object_class
             $row_fields);
 
         $configurable->conditional_control($this->name().'/action', $this->name().'/action');
+        
         // Help specified ?
         if($this->documentation()){
-            $configurable->set_field_help($this->name().'/action', $this->documentation());
+            //add xml help msg at the top
+            $all_help_msg =  array_merge($this->documentation(), $all_help_msg);
+        }
+        if($all_help_msg){
+            $configurable->set_field_help($this->name().'/action', $all_help_msg);
         }
         // Associated category ?
         if($this->category()){
@@ -411,6 +450,7 @@ class Safe_audit_point_system_load extends Safe_audit_point_system
         parent::__construct($module, Safe_audit_point::RESOURCE_LOADAVG);
         $this->set_description('Load Average');
         $this->set_category('System');
+        
         // Create process check fields
         $this->_value_fields = array(
             Safe_audit_point::ID_1MIN => array(
@@ -418,21 +458,35 @@ class Safe_audit_point_system_load extends Safe_audit_point_system
                         'type'  => 'numeric',
                         'size'  => '5',
                         'default' => '50',
+                        'help' => 'System load average threshold for ' . Safe_audit_point::ID_1MIN . '.'
                     ),
             Safe_audit_point::ID_5MIN => array(
                         'label' => Safe_audit_point::ID_5MIN,
                         'type'  => 'numeric',
                         'size'  => '5',
                         'default' => '50',
+                        'help' => 'System load average threshold for ' . Safe_audit_point::ID_5MIN . '.'
                     ),
             Safe_audit_point::ID_15MIN => array(
                         'label' => Safe_audit_point::ID_15MIN,
                         'type'  => 'numeric',
                         'size'  => '5',
                         'default' => '50',
+                        'help' => 'System load average threshold for  ' . Safe_audit_point::ID_15MIN . '.'
                     ),
                 );
     }
+    public function set_documentation($documentation)
+    {
+        $cpu_number = $this->_module->node()->os()->cpu_count();
+        $msg = " Your system have {$cpu_number} cpu.";
+        if(is_array($documentation)){
+            $documentation[] = $msg;
+        }else{
+            $documentation = $documentation . $msg;
+        }
+        parent::set_documentation($documentation);
+    }
     protected function _monit_condition()
     {
         $cfg = parent::_monit_condition();
@@ -464,18 +518,21 @@ class Safe_audit_point_system_cpu extends Safe_audit_point_system
                         'type'  => 'numeric',
                         'size'  => '5',
                         'default' => '50',
+                        'help' => 'CPU Usage threshold for ' . Safe_audit_point::ID_USER . '.'
                     ),
             Safe_audit_point::ID_SYSTEM => array(
                         'label' => ucfirst(Safe_audit_point::ID_SYSTEM),
                         'type'  => 'numeric',
                         'size'  => '5',
                         'default' => '50',
+                        'help' => 'CPU Usage threshold for ' . Safe_audit_point::ID_SYSTEM . '.'
                     ),
             Safe_audit_point::ID_WAIT => array(
                         'label' => ucfirst(Safe_audit_point::ID_WAIT),
                         'type'  => 'numeric',
                         'size'  => '5',
                         'default' => '50',
+                        'help' => 'CPU Usage threshold for ' . Safe_audit_point::ID_WAIT . '.'
                     ),
                 );
     }
@@ -678,7 +735,7 @@ class Safe_audit_point_filesystem extends Safe_audit_point
         $type = Safe_audit_point::TYPE_FILESYSTEM;
         parent::__construct($module, $type, $id);
         $this->set_category('System');
-        $this->set_description('File System Usage %');
+        $this->set_description('File System Usage % ('.$id.')');
         $this->_path=$path;
     }
     public function set_parameters($parameters)
@@ -723,12 +780,14 @@ class Safe_audit_point_process extends Safe_audit_point
                         'type'  => 'numeric',
                         'size'  => '5',
                         'default' => '50',
+                        'help' => 'Memory Usage threshold for ' . $this->description() . '.'
                     ),
             'cpu' => array(
                         'label' => 'CPU %',
                         'type'  => 'numeric',
                         'size'  => '5',
                         'default' => '50',
+                        'help' => 'CPU Usage threshold for ' . $this->description() . '.'
                     ),
             'restart' => array(
                         'label' => 'Auto Restart',
