@@ -84,6 +84,7 @@ class Safe_configuration_manager_class extends Safe_object_class
     private $_directories;
     private $_configs = array();
     private $_services = array();
+    private $_audits = array();
     private $_license = array();
     private $_about = array();
     private $_update = array();
@@ -102,19 +103,65 @@ class Safe_configuration_manager_class extends Safe_object_class
         // Check services
         if (isset($prod_def['services'])) {
             $services = $prod_def['services'];
+            
             // Section exists check if some services defined
             if (isset($services['service'])) {
-                // Check if
-                if (!is_indexed_array($services['service'])) {
-                    $tmp = $services['service'];
-                    unset($services['service']);
-                    $services['service'] = array(
-                        '0' => $tmp
-                    );
-                }
+                $services['service'] = to_index_array($services['service']);
             }
             $this->_services = $services['service'];
         }
+        // Process audit configuration if any
+        if (isset($prod_def['audit'])) {
+            $_audits = array();
+            $audits = $prod_def['audit']['modules'];
+            // Section exists check if some modules defined
+            if (isset($audits['module'])) {
+                $audits['module'] = to_index_array($audits['module']);
+                // Arrange to array
+                foreach($audits['module'] as $mod){
+                    $general = $mod['@attributes'];
+                    $name = $general['name'];
+                    unset($general['name']);
+                    if($name){
+                        $points = to_index_array($mod['point']);
+                        foreach($points as $point){
+                            $point_def = array();
+                            $point_name = $point['@attributes']['name'];
+                            if(!$point_name){
+                                $point_name = $point['@attributes']['type'];
+                            }
+                            //unset($point['@attributes']['name']);
+                            foreach($point['@attributes'] as $k => $v){
+                                $point_def[$k] = $v;
+                            }
+                            // loop on param
+                            foreach(array('arg', 'param') as $t){
+                                if($point[$t]){
+                                    $param = to_index_array($point[$t]);
+                                    foreach($param as $p){
+                                        $pname = $p['@attributes']['name'];
+                                        unset($p['@attributes']['name']);
+                                        if($pname){
+                                            foreach($p['@attributes'] as $k => $v){
+                                                $point_def[$t][$pname][$k] = $v;
+                                            }
+                                        }
+                                    }
+                                }
+                            }
+                            if($point['help']){
+                                $point_def['help'] = $point['help'];
+                            }
+                            if($point_def){
+                                $_audits[$name][$point_name] = $point_def;
+                            }
+                        }
+                    }
+                }
+                $this->_audits = $_audits;
+            }
+        }
+        // check audit parameters
         // Make sure configuration,log directories exist and webconfig
         // can play with it
         if (isset($this->_directories['configuration'])) $this->_node->mkdir($this->_directories['configuration']);
@@ -122,6 +169,8 @@ class Safe_configuration_manager_class extends Safe_object_class
             $this->_node->mkdir($this->_directories['log'] , false);
         if (isset($this->_directories['core-dump']))
             $this->_node->mkdir($this->_directories['core-dump'] , false);
+        if (isset($this->_directories['data']))
+            $this->_node->mkdir($this->_directories['data'] , false);
         // License
         $this->_license['file'] = $prod_def['license'];
         $this->_license['pubkey'] = $prod_def['pubkey'];
@@ -140,23 +189,12 @@ class Safe_configuration_manager_class extends Safe_object_class
     {
         $this->_configs[] = $config;
     }
-    public function add_error($action, $target, $fix_url = null, $fix_url_label = null, $fix_url_value = null)
+    public function add_error($error = array())
     {
         if (!$this->_forget_error) {
-            $this->_errors[] = array(
-                'action' => $action,
-                'target' => $target,
-                'fix_url' => $fix_url,
-                'fix_url_label' => $fix_url_label,
-                'fix_url_value' => $fix_url_value
-            );
-            if (isset($fix_url) && strlen($fix_url)) $url = safe_form_with_submit_button('', $fix_url, (isset($fix_url_label)) ? $fix_url_label : 'Fix it !', (isset($fix_url_value)) ? $fix_url_value : 'fix_it');
-            else $url = 'ERROR';
-            $this->_progress[] = array(
-                $action,
-                $target,
-                $url
-            );
+            $this->_errors[] = $error;
+            $error['type'] = 'ERROR';
+            $this->_progress[] = $error;
         }
     }
     public function reset_errors()
@@ -185,10 +223,17 @@ class Safe_configuration_manager_class extends Safe_object_class
     {
         return $this->_directories;
     }
-    public function directory($type)
+    public function directory($type, $default=null, $fatal=true)
     {
-        if (isset($this->_directories[$type])) return $this->_directories[$type];
-        else throw new Exception('Undefined directory (' . $type . ')');
+        if (isset($this->_directories[$type])){
+            return $this->_directories[$type];
+        }else if(null != $default){
+            return $default;
+        }
+        if($fatal){
+            throw new Exception('Undefined directory (' . $type . ')');
+        }
+        return false;
     }
     public function about()
     {
@@ -198,6 +243,10 @@ class Safe_configuration_manager_class extends Safe_object_class
     {
         return $this->_services;
     }
+    public function audits()
+    {
+        return $this->_audits;
+    }
     public function license()
     {
         return $this->_license;
@@ -369,14 +418,19 @@ class Safe_configuration_manager_class extends Safe_object_class
             }
             // Report error
             $this->_progress[] = array(
-                $config->type() . ' ' . $config->action() ,
-                $display_action,
-                (true == $success) ? 'OK' : 'ERROR'
+                    'module' => 'Configuration Manager',
+                    'obj_type' => $config->type() . ' ' . $config->action(),
+                    'description' => $display_action,
+                    'type' => (true == $success) ? 'OK' : 'ERROR'
             );
-            error_log('write_config - ' .
-                $config->type() . ' ' . $config->action() . ' - ' .
-                $display_action . ' - ' . ((true == $success) ? 'OK' : 'ERROR'));
-            if (true != $success) return false;
+            if(true != $sucess){
+                error_log('write_config - ' .
+                    $config->type() . ' ' . $config->action() . ' - ' .
+                    $display_action . ' - ' . ((true == $success) ? 'OK' : 'ERROR'));
+            }
+            if (true != $success){
+                return false;
+            }
         }
         // Update configuration status
         $this->_update_status();
@@ -403,6 +457,15 @@ class Safe_configuration_manager_class extends Safe_object_class
         $data = $this->_get_status();
         return $data['generate']['count'];
     }
+    public function generate_status()
+    {
+        $data = $this->_get_status();
+        if($data){
+            return $data['generate'];
+        }else{
+            return array('count' => 0);
+        }
+    }
 
     public function node()
     {
@@ -417,16 +480,18 @@ class Safe_configuration_manager_class extends Safe_object_class
         }
         return $this->_archive_manager;
     }
-    public function checklist(&$list)
+    public function checklist(&$list, $check_generate=true)
     {
-        $status = $this->_get_status();
-        if(!$status['generate']['count']){
-            $list[] = array(
-                'module' =>'application',
-                'obj_type' =>'configuration',
-                'description' => 'Not generated',
-            );
+        if($check_generate){
+            $status = $this->_get_status();
+            if(!$status['generate']['count']){
+                $list[] = array(
+                    'module' =>'application',
+                    'obj_type' =>'configuration',
+                    'description' => 'Not generated',
+                );
 
+            }
         }
     }
 }
@@ -457,40 +522,29 @@ class Safe_archive_manager_class extends Safe_object_class
     {
         error_log('restore_archive - '.$archive_name);
         $os = $this->_node->os();
-        $cmd = '/bin/tar -xpsvzf ';
-        $cmd.= $this->_arch_dir . '/' . $archive_name . ' -C /';
+        $cmd = 'nohup /usr/local/sng/bin/sng-archive-restore '.$this->_arch_dir.'/'.$archive_name;
         $output = array();
-        $rc = $os->execute($cmd, $output, false, true);
+        $rc = $os->execute($cmd, $output, true, true);
         return $rc;
     }
-    public function create_archive()
+    public function create_archive($archive_type = 'system')
     {
-        $backup_conf = array();
-        $os = $this->_node->os();
-        $rc = $os->read_file('/etc/backup.conf', $backup_conf, true);
-        // Clean and adjust file list
-        $files_list = array();
-        foreach ($backup_conf as $pattern) {
-            $pattern = trim($pattern);
-            if ('#' != $pattern[0]) {
-                $files = $os->search_files($pattern);
-                foreach ($files as $file) {
-                    $files_list[] = $file;
-                }
-            }
+        if(empty($archive_type)){
+            $archive_type = 'system';
         }
+        $os = $this->_node->os();
         // Ready to ZIIIIP....
-        $name = php_uname('n') . '-backup-' . date('Y-m-d-H-i-s') . '.tgz';
+        $name = php_uname('n') . '-'.$archive_type.'-backup-' . date('Y-m-d-H-i-s') . '.tgz';
         error_log('create_archive - '.$name);
         // Create tar command line
-        $cmd = '/bin/tar -cpsvzf ';
-        $cmd.= $this->_arch_dir . '/tmp/' . $name . ' ';
-        $cmd.= implode(' ', $files_list);
+        $cmd = 'nohup /usr/local/sng/bin/sng-archive-backup '.$archive_type.' '.$this->_arch_dir.'/tmp/'.$name;
         $output = array();
-        $rc = $os->execute($cmd, $output, false, true);
+        $rc = $os->execute($cmd, $output, true, true);
         if ($rc) {
             // Succeed move archive to vault
-            if ($os->move_file($this->_arch_dir . '/tmp/' . $name, $this->_arch_dir . '/' . $name)) return $name;
+            if ($os->move_file($this->_arch_dir . '/tmp/' . $name, $this->_arch_dir . '/' . $name)){
+                return $name;
+            }
         }
         return false;
     }
@@ -499,5 +553,40 @@ class Safe_archive_manager_class extends Safe_object_class
         if ($pattern) return $this->_arch_dir . '/*.tgz';
         else return $this->_arch_dir;
     }
+    public function info($archive_name)
+    {
+        // Check if specified archive can be restored
+        $os = $this->_node->os();
+        $cmd = 'nohup /usr/local/sng/bin/sng-archive-info '.$this->_arch_dir.'/'.$archive_name;
+        $output = array();
+        $rc = $os->execute($cmd, $output, true, true);
+        foreach($output as $line){
+            $t = explode('=', $line);
+            $info[$t[0]] = $t[1];
+        }
+
+        return $info;
+    }
+    public function can_restore($archive_name, &$reasons)
+    {
+        // Check if specified archive can be restored
+        $os = $this->_node->os();
+        $cmd = 'nohup /usr/local/sng/bin/sng-archive-test '.$this->_arch_dir.'/'.$archive_name;
+        $rc = $os->execute($cmd, $reasons, true, true);
+        return $rc;
+    }
+    public function get_archive_configurable_object()
+    {
+        $os = $this->_node->os();
+        $conf_file_list =  $os->search_files('/etc/*-backup.conf');
+        $type = array();
+        foreach($conf_file_list as $file_name){
+             $tmp_type= basename($file_name, '-backup.conf');
+             $type[$tmp_type] = ucfirst($tmp_type).' '.'Backup';
+        }
+        $param =  new Safe_configurable_object_class($this->_config_manager, 'archive');
+        $param->add_enum_field('backup_type', 'Backup Type', 'dropdown', '', $type);
+        return $param;
+    }
 }
 /* End of file safe_config_manager_class.php */
