@@ -110,6 +110,7 @@ class Safe_form_class
      * @var array (strinf $name => string $value)
      */
     private $hiddens = array();
+    private $privates = array();
     /**
      * Configuration of each <input>s
      * Feild infos, validation infos and <input> attributes
@@ -205,6 +206,10 @@ class Safe_form_class
         }
         return false;
     }
+    public function clear_rules(){
+        $this->CI->form_validation->clear_rules($c);
+        $this->rules_setted = false;
+    }
     /**
      * Set rules of Codeignter formvalidation
      *
@@ -270,7 +275,7 @@ class Safe_form_class
         if (!$this->cc_setted) {
             $c = $this->config;
             foreach ($c as $k => $v) {
-                if (!isset($v['group'])) {
+                if (!isset($v['group']) || !isset($v['group_condition']) ) {
                     $v['group'] = '';
                 }
             }
@@ -298,8 +303,10 @@ class Safe_form_class
                 if ($v['type'] == 'hidden') {
                     isset($v['value']) && $this->hiddens[$v['field']] = $v['value'];
                     unset($this->config[$k]); // not necessary
-                    
-                } else {
+                }elseif($v['type'] == 'private'){
+                    isset($v['value']) && $this->privates[$v['field']] = $v['value'];
+                    unset($this->config[$k]); // not necessary
+                }else {
                     array_push($c, $v);
                 }
             }
@@ -325,6 +332,7 @@ class Safe_form_class
         }
         $this->config = $c;
         $this->fix_config();
+        $this->clear_rules();
         $this->set_rules();
     }
     /**
@@ -337,7 +345,7 @@ class Safe_form_class
         // return
         // form_open($this->action,$this->form_attributes,$this->hiddens)."\n";
         // i trust the user so the uri is not rewritten by codigniter...
-        $html = form_open('__://__', $this->form_attributes, $this->hiddens) . "\n";
+        $html = form_open('__://__', $this->form_attributes, $this->hiddens + $this->privates) . "\n";
         $html = preg_replace('#__://__#', $this->action, $html);
         return $html;
     }
@@ -353,7 +361,7 @@ class Safe_form_class
         // i trust the user so the uri is not rewritten by codigniter...
         $html = form_open_multipart('__://__', array(
             "enctype" => "multipart/form-data"
-        ) , $this->hiddens) . "\n";
+        ) , $this->hiddens + $this->privates) . "\n";
         $html = preg_replace('#__://__#', $this->action, $html);
         return $html;
     }
@@ -431,15 +439,15 @@ class Safe_form_class
                         //multiple values with multiple selections
                         foreach ($curr_conf['value'] as $key => $value) {
                             foreach ($curr_conf['default'] as $default) {
-                                if ($value == $default) {
+                                if ($key == $default) {
                                     $def = true;
                                     break;
                                 } else {
                                     $def = false;
                                 }
                             }
-                            if (set_checkbox($curr_conf['field'], $value, $def)) {
-                                $temp_conf['checked'][$value] = "checked";
+                            if (set_checkbox($curr_conf['field'], $key, $def)) {
+                                $temp_conf['checked'][$key] = "checked";
                             }
                         }
                     } else {
@@ -459,6 +467,8 @@ class Safe_form_class
 
             case 'hidden':
                 break;
+            case 'private':
+                    break;
 
             default:
                 $temp_conf['value'] = set_value($curr_conf['field'], (isset($curr_conf['default'])) ? $curr_conf['default'] : null);
@@ -491,10 +501,10 @@ class Safe_form_class
                 $checkboxes = array();
                 if (is_array($temp_conf['value'])) {
                     foreach ($temp_conf['value'] as $key => $value) {
-                        if (isset($temp_conf['checked']) && isset($temp_conf['checked'][$value])) {
-                            $checkboxes[] = $prefix . form_checkbox($temp_conf['name'], $value, $temp_conf['checked'][$value], '') . ' ' . htmlspecialchars($value) . $suffix;
+                        if (isset($temp_conf['checked']) && isset($temp_conf['checked'][$key])) {
+                            $checkboxes[] = $prefix . form_checkbox($temp_conf['name'], $key, $temp_conf['checked'][$key], '') . ' ' . htmlspecialchars($value) . $suffix;
                         } else {
-                            $checkboxes[] = $prefix . form_checkbox($temp_conf['name'], $value, '', '') . ' ' . htmlspecialchars($value) . $suffix;
+                            $checkboxes[] = $prefix . form_checkbox($temp_conf['name'], $key, '', '') . ' ' . htmlspecialchars($value) . $suffix;
                         }
                     }
                 }
@@ -518,17 +528,33 @@ class Safe_form_class
                     // onchange call the js function safe_disabled using the group name and its current value
                     $conditional_control = 'onchange="safe_disabled(\'' . $temp_conf['group'] . '\',this.value);"';
                 }
-                if (isset($curr_conf['disabled'])) $conditional_control.= ' disabled="' . $curr_conf['disabled'] . '"';
+                if (!empty($temp_conf['group_condition'])) {
+                    // onchange call the js function safe_condition_disabled using  its name and its current value
+                    $conditional_control = 'onchange="safe_condition_disabled(this.name,this.value);"';
+                }
+                if (isset($curr_conf['disabled'])){
+                    $conditional_control.= ' disabled="' . $curr_conf['disabled'] . '"';
+                    $conditional_control.= ' class="field_'.$curr_conf['disabled'].'"';
+                }
+                if (isset($temp_conf['style'])) $conditional_control.= ' style="' . $temp_conf['style'] . '"';
+                unset($temp_conf['style']);
                 unset($temp_conf['name']);
                 unset($temp_conf['value']);
                 unset($temp_conf['type']);
                 $extra = '';
                 foreach ($temp_conf as $k => $v) $extra.= $k . '="' . htmlentities($v) . ' ';
                 //var_dump($temp_conf);
+                /*
+                $_options = array();
+                foreach($curr_conf['value'] as $opt){
+                    $_options[] = htmlentities($opt);
+                }
+                 */
                 $_options = $curr_conf['value'];
                 $_selected = $curr_conf['default'];
                 $_option_error = false;
                 // Check if default specified and exists in the possible values
+                
                 if ($curr_conf['default']) {
                     if (!isset($curr_conf['value'][$_selected])) {
                         $_options[$_selected] = 'INVALID';
@@ -539,12 +565,14 @@ class Safe_form_class
                 $_select_html = $prefix . form_dropdown($curr_conf['field'], $_options, $_selected, $conditional_control) . $suffix;
                 // Add some extra style to invalid enum if exists
                 if ($_option_error) {
-                    $_select_html = preg_replace('/value=\"' . $_selected . '\"/', 'value="' . $_selected . '" style="color:red; font-weight:bolder"', $_select_html);
+                    $_select_html = str_replace('value="' . $_selected . '"', 'value="' . $_selected . '" style="color:red; font-weight:bolder"', $_select_html);
                     $_select_html = '<span class="field_error"><p>The field contains an invalid option.</p></span>' . $_select_html;
                 }
                 return $_select_html;
             case 'hidden':
                 break;
+            case 'private':
+                    break;
 
             case 'date':
             case 'datetime':
@@ -554,14 +582,46 @@ class Safe_form_class
                 $temp_conf['type'] = 'text';
             default:
                 $_extra = '';
-                if('password' == $curr_conf['type']){
-                    $_extra = ' autocomplete="off"';
-                }
+                // Disabled ?
                 if (isset($curr_conf['disabled'])){
-                    return $prefix . form_input($temp_conf, '', $curr_conf['disabled']) . $suffix;
-                } else{
-                    return $prefix . form_input($temp_conf, '', $_extra) . $suffix;
+                    $_extra .= ' disabled="'.$curr_conf['disabled'].'"';
+                    $_extra .= ' class="field_'.$curr_conf['disabled'].'"';
                 }
+                // Password ?
+                if('password' == $curr_conf['type']){
+                    $_extra .= ' autocomplete="off"';
+                }
+                $_input_type = array(
+                    "button",
+                    "checkbox",
+                    "color",
+                    "date",
+                    "datetime",
+                    "datetime-local",
+                    "email",
+                    "file",
+                    "hidden",
+                    "image",
+                    "month",
+                    "number",
+                    "password",
+                    "radio",
+                    "range",
+                    "reset",
+                    "search",
+                    "submit",
+                    "tel",
+                    "text",
+                    "time",
+                    "url",
+                    "week",
+                    );   
+                // Remove type from conf if not known input type (like string, 
+                // integer...)
+                if(!in_array($temp_conf['type'], $_input_type)){
+                    unset($temp_conf['type']);
+                }
+                return $prefix . form_input($temp_conf, '', $_extra) . $suffix;
             } // end of switch
             
         }
@@ -577,6 +637,7 @@ class Safe_form_class
     {
         if (!empty($this->config)) {
             $c = $this->config[$this->current_field]; // current config
+            
             $d['name'] = $c['field'];
             switch ($c['type']) {
             case "radio":
@@ -590,7 +651,18 @@ class Safe_form_class
                     }
                 }
             }
-            if (isset($c['label']) && $c['label']) return $prefix . form_label($c['label'], $c['field'], $this->label_attributes) . $suffix;
+            if (isset($c['label']) && $c['label']) {
+                if(isset($c['label_style'])){
+                    $this->label_attributes['style'] = 'display:inline-block;' . $c['label_style'];
+                    $this->label_attributes['align'] = 'right';
+                } 
+                $tmp = $prefix . form_label($c['label'], $c['field'], $this->label_attributes) . $suffix;
+                if(isset($c['label_style'])){
+                    unset($this->label_attributes['style']);
+                    unset($this->label_attributes['align']);
+                }
+                return $tmp;
+            }
             else return $prefix . " " . $suffix;
         }
     }
@@ -658,7 +730,18 @@ class Safe_form_class
     //    if (! empty ( $post ))
     //      ;
     //  }
+    public function set_message($msg_array){
+        foreach($msg_array as $key => $message){
+            $this->CI->form_validation->set_message($key, $message);
+        }
+    }
     
+    public function set_validation_obj($obj){
+         $this->CI->form_validation->set_validation_obj($obj);
+    }
+    public function set_error($field ='', $message =''){
+        $this->CI->form_validation->set_error($field , $message);
+    }
     /**
      * excute the Codeignter form_validation->run();
      */
