@@ -488,16 +488,17 @@ class Safe_module_class extends Safe_object_class
             if (isset($data_key)) {
                 $tmp_file_name = pathinfo($data_key, PATHINFO_FILENAME);
                 if($data_cfg[$data_name]['file_ext']){
-                    $tmp_ext = $data_cfg[$data_name]['file_ext'];
+                    $tmp_ext = 
+                        '.{'.str_replace('|',',',$data_cfg[$data_name]['file_ext']).'}';
                 }else{
-                    $tmp_ext = pathinfo($data_cfg[$data_name]['pattern'] , PATHINFO_EXTENSION);
+                    $tmp_ext = '.'.pathinfo($data_cfg[$data_name]['pattern'] , PATHINFO_EXTENSION);
                 }
                 if($data_cfg[$data_name]['file_dir']){
                     $tmp_file_path = $data_cfg[$data_name]['file_dir'];
                 }else{
                     $tmp_file_path = pathinfo($data_cfg[$data_name]['pattern'] , PATHINFO_DIRNAME );
                 }
-                $file = $this->node()->os()->find_file($tmp_file_path, $tmp_file_name.'.'.$tmp_ext, $data_cfg[$data_name]['recursive']);
+                $file = $this->node()->os()->find_file($tmp_file_path, $tmp_file_name.$tmp_ext, $data_cfg[$data_name]['recursive']);
                 if($file){
                     $files[] = $file;
                 }
