@@ -45,6 +45,12 @@ class Safe_object_serializer_class
         $ser = Safe_object_serializer_class::get_serializer();
         return $ser->next_insert_id();
     }
+    public static function build_unique_name($prefix)
+    {
+        $prefix =  preg_replace('/[^0-9a-zA-Z\-_]/', '', $prefix);
+        $id = Safe_object_serializer_class::get_next_unique_id();
+        return $prefix.'_'.$id;   
+    }
     /**
      * @var CI_DB_pdo_driver
      */
@@ -60,6 +66,7 @@ class Safe_object_serializer_class
     // TODO: put some method private and proteceted.
     public function __construct()
     {
+        safe_enter_critical_section();
         try {
             $CI = & get_instance();
             // if the files for the db doesn't it will be created.
@@ -70,9 +77,10 @@ class Safe_object_serializer_class
             // TODO: catch CodeIgniter DB_Driver exception.
             // we will never go here since CodeIgniter DB_Driver will fail
             // with a PHP error on the user browser
+            error_log("ERROR - Cannot open database ".$e->getMessage());
             show_error(__FUNCTION__ . ": can't open databases");
-            return false;
         }
+        safe_leave_critical_section();
     }
     public function __destruct()
     {
@@ -89,7 +97,7 @@ class Safe_object_serializer_class
     private function create_db()
     {
         try {
-            error_log("Safe_object_serializer: Trying to creating safe_object DB");
+            error_log("Safe_object_serializer: Trying to create safe_object DB");
             $sqlq = "
         CREATE TABLE '" . $this->db_table . "' (
           'id' INTEGER PRIMARY KEY AUTOINCREMENT,
@@ -102,28 +110,42 @@ class Safe_object_serializer_class
           'data' TEXT,
           'modified_date' DATE DEFAULT (datetime('now','localtime')),
           'modified_by'   VARCHAR(128),
+          'data_ref' TEXT,
           CONSTRAINT unique_object UNIQUE ('type', 'path', 'name') ) ;
       ";
             if (!$this->db->query($sqlq)) throw new Exception("Cannot create DB!");
         }
         catch(Exception $e) {
-            safe_add_error(__FUNCTION__ . ":{$this->filename}: " . $e->getMessage() , true);
+            //safe_add_error(__FUNCTION__ . ":{$this->filename}: " . $e->getMessage() , true);
+            error_log("ERROR - Cannot create_db() ".$e->getMessage());
             return false;
         }
         return true;
     }
-    public function store_object($path, $name, $class, $object_data, $object_type = NULL, &$object_status = NULL, &$object_id = NULL)
+    public function store_object($path, $name, $class, $object_data, $object_type = NULL, &$object_status = NULL, &$object_id = NULL, $reuse=true)
     {
         $insert['path'] = $path;
         $insert['name'] = $name;
         $insert['class'] = $class;
         $insert['data'] = json_encode($object_data);
         $insert['modified_date'] = date('Y-m-d H:i:s');
+        if(isset($_SESSION['user_login'])){
+            $insert['modified_by'] = $_SESSION['user_login'];
+        }
         if (isset($object_type)) $insert['type'] = $object_type;
         // Object exists ?
         $old_data = array();
         $obj_status = '';
-        if (TRUE == $this->retrieve_object($path, $name, $old_data, $obj_status, $object_id)) {
+        $existing_obj = $this->retrieve_object($path, $name, $old_data, $obj_status, $object_id);
+        // Object exists and allowed to be re-used ?
+        // OR
+        // Object exists and NOT allowed to be re-used and not yet serialized (ie. NEW) ?
+        if (
+            (TRUE == $reuse && TRUE == $existing_obj) 
+            ||
+            (TRUE != $reuse && TRUE == $existing_obj && $obj_status == Safe_object_serializer_class::OBJ_STATUS_NEW)
+           )
+        {
             // Yes, need to update
             if (json_encode($old_data) != $insert['data']) {
                 // Data modified, update data and status to modified
@@ -157,6 +179,21 @@ class Safe_object_serializer_class
             }
             if (isset($object_status)) $object_status = $obj_status;
         } else {
+            // Existing and not allowed to be re-used
+            // yes, rename old record
+            if (true != $reuse && true == $existing_obj){
+                $update = $insert;
+                $update['name'] = $this->build_unique_name('__deleted__'.$name);
+                $update['modified'] = 1;
+                $update['status'] = Safe_object_serializer_class::OBJ_STATUS_DELETED;
+                unset($update['data']);
+                $this->db->where('id', $object_id);
+                if (!$this->db->update($this->db_table, $update)) {
+                    error_log("Fail to UPDATE to DEL serialize object " . $name());
+                    return FALSE;
+                }
+            }
+
             // No, insert it
             if (!$this->db->insert($this->db_table, $insert)) {
                 error_log("Fail to INSERT serialize object " . $name());
@@ -175,13 +212,16 @@ class Safe_object_serializer_class
      *           
      * @return
      */
-    public function dispose_object($path, $name)
+    public function dispose_object($path, $name, $reuse=true)
     {
         // If NEW object then delete it
         // else mark as deleted (and modified)
         $dummy = array();
         $status = Safe_object_serializer_class::OBJ_STATUS_MODIFIED;
         $id = 0;
+        if(isset($_SESSION['user_login'])){
+            $insert['modified_by'] = $_SESSION['user_login'];
+        }
         // Retrieve object status and id
         if ($this->retrieve_object($path, $name, $dummy, $status, $id)) {
             if ($status == Safe_object_serializer_class::OBJ_STATUS_NEW) {
@@ -192,6 +232,11 @@ class Safe_object_serializer_class
                 // Existing and fetched object, mark as deleted
                 $insert['modified'] = 1;
                 $insert['status'] = Safe_object_serializer_class::OBJ_STATUS_DELETED;
+                if(true != $reuse){
+                    // We'd like this object to be gone for good so cannot be 
+                    // updated after delete
+                    $insert['name'] = Safe_object_serializer_class::build_unique_name('__deleted__'.$name);
+                }
                 $this->db->where('id', $id);
                 if (!$this->db->update($this->db_table, $insert)) {
                     error_log("Fail to UPDATE serialize object " . $name());
@@ -229,6 +274,33 @@ class Safe_object_serializer_class
         }
         return FALSE;
     }
+    /**
+     * @brief Mark an object as modified
+     *
+     * @param[in out] $path
+     * @param[in out] $name
+     *
+     * @return
+     */
+    public function mark_object_modified($path, $name)
+    {
+        $dummy = array();
+        $status = Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE;
+        $id = 0;
+        // Retrieve object status and id
+        if ($this->retrieve_object($path, $name, $dummy, $status, $id)) {
+            // Existing and fetched object, mark as deleted
+            $insert['modified'] = 1;
+            $insert['status'] = Safe_object_serializer_class::OBJ_STATUS_MODIFIED;
+            $this->db->where('id', $id);
+            if (!$this->db->update($this->db_table, $insert)) {
+                error_log("Fail to UPDATE serialize object " . $name());
+                return FALSE;
+            }
+            return TRUE;
+        }
+        return FALSE;
+    }
     /**
      * @brief
      *           
@@ -441,6 +513,7 @@ class Safe_object_class
     private $_documentation = '';
     private $_class = "";
     private $_aggr_obj_type = array();
+    protected $_properties = array();
     public function __construct($path, $name)
     {
         $this->_path = $path;
@@ -474,6 +547,7 @@ class Safe_object_class
     {
         return TRUE;
     }
+
     /**
      * @brief Return object description
      *         
@@ -494,6 +568,18 @@ class Safe_object_class
     {
         $this->_description = $description;
     }
+    public function set_properties($properties)
+    {
+        $this->_properties = $properties;
+    }
+    public function property($name, $default)
+    {
+        if($this->_properties[$name]){
+            return $this->_properties[$name];
+        }else{
+            return $default;
+        }
+    }
     /**
      * @brief Return object Documentation
      *
@@ -516,24 +602,6 @@ class Safe_object_class
     {
         $this->_documentation = $documentation;
     }
-    /**
-     * @brief REST GET data
-     *         
-     * @return
-     */
-    public function rest_get_data()
-    {
-        $data = array();
-        // Check for aggregate objects
-        $aggr_obj_list = $this->has_aggregate_objects();
-        if ($aggr_obj_list) {
-            foreach ($aggr_obj_list as $aggr_obj_type => $aggr_obj_name) {
-                $agg_objs = $this->get_aggregate_objects($aggr_obj_type);
-                foreach ($agg_objs as $obj) $data[$aggr_obj_type][$obj->name() ] = $obj->rest_get_data();
-            }
-        }
-        return $data;
-    }
     /**
      * Used to tell application configuration manager that this is an aggregated objects
      * @return array ( 'type' => object definition )
@@ -621,6 +689,26 @@ class Safe_object_class
         if(!$definition['dynamic']){
             $definition['dynamic'] = false;
         }
+        if(!$definition['base_path'] && false !== $definition['base_path']){
+            $definition['base_path'] = $this->object_name().'/'.$type;
+        }
+        if(!isset($definition['configurable'])){
+            $definition['configurable'] = true;
+        }
+        if(!isset($definition['pagination'])){
+            $definition['pagination'] = false;
+        }
+        if(!isset($definition['global_methods'])){
+            $definition['global_methods'] = false;
+        }
+        if(isset($definition['methods']) && is_array($definition['methods'])){
+            foreach($definition['methods'] as $method){
+                if(isset($method['scope']) && ($method['scope'] == 'global' || $method['scope'] == 'both')){
+                    $definition['global_methods'] = true;
+                }
+            }
+        }
+        
         // ...
         /*
         echo '<pre>';
@@ -638,16 +726,18 @@ class Safe_object_class
      *                			Array of aggregated configs
      */
     private $_agg_objects_cache = array();
-    public function get_aggregate_objects($type, $all=false)
+    public function get_aggregate_objects($type, $all = false, $params = array())
     {
         $definition = $this->_aggr_obj_type[$type];
+        
         // Do we have base path ?
         if($definition['base_path']) {
             // Check objects cache (or requesting all)
             if(!$this->_agg_objects_cache[$type] || $this->_agg_objects_cache[$type]['all'] != $all){
+                $objects = array();
                 // Re-create object list
-                $objects = Safe_object_serializer_class::get_serializer()->find_objects($definition['base_path']);
-                foreach ($objects as $k => $v) {
+                $_objects = Safe_object_serializer_class::get_serializer()->find_objects($definition['base_path']);
+                foreach ($_objects as $k => $v) {
                     $obj = new $v['class']($this->node(), $definition['base_path'], $k);
                     $obj->configure();
                     $obj->synch();
@@ -656,8 +746,8 @@ class Safe_object_class
                 // Check if we want the deleted objects as well
                 if($all){
                     // Re-create object list
-                    $deleted = Safe_object_serializer_class::get_serializer()->find_deleted_objects($definition['base_path']);
-                    foreach ($deleted as $k => $v) {
+                    $_deleted = Safe_object_serializer_class::get_serializer()->find_deleted_objects($definition['base_path']);
+                    foreach ($_deleted as $k => $v) {
                         $obj = new $v['class']($this->node(), $definition['base_path'], $k);
                         $obj->configure();
                         $obj->unserialize();
@@ -674,13 +764,14 @@ class Safe_object_class
             return call_user_func(array(
                 $this,
                 'api_retrieve_'.$type
-            ));
+            ), null, $params);
         } else if (method_exists($this, $type)) {
             return call_user_func(array(
                 $this,
                 $type
             ));
         }
+        error_log("WARNING - Cannot retrieve aggregate_object ${type} for module ".$this->name());
         return array();
     }
     
@@ -718,10 +809,15 @@ class Safe_object_class
         if($objs[$name]){
             $reason = "Already exists";
             return false;
+        }else{
+            if (preg_match('/^__.*__$/i', $name)) {
+                $reason = " '{$name}' is reserved by system.";
+                return false;
+            }
         }
         return true;
     }
-    public function create_aggregate_object($type, $name='default')
+    public function create_aggregate_object($type, $name='default', &$output=null)
     {
         if('configuration'==$type && $this->config()) {
             return $this->config();
@@ -732,18 +828,50 @@ class Safe_object_class
         // Create allowed ?
        // if($definition['methods']['create'])
         {
+            $data = null;
             $create_name = 'api_create_'.$type;
             if(method_exists($this, $create_name)) {
                 $obj =  call_user_func_array(
                     array(
                     $this,
                     $create_name),
-                    array($name)
+                    array($name , $data , &$output)
                 );
             }
         }
         return $obj;
     }
+    public function delete_aggregate_object($type, $name='default')
+    {
+        $definition = $this->_aggr_obj_type[$type];
+        // delete allowed ?
+        if($definition['methods']['delete']){
+            $delete_name = 'api_delete_'.$type;
+            if(method_exists($this, $delete_name)) {
+                return  call_user_func_array(
+                        array(
+                                $this,
+                                $delete_name),
+                        array($name)
+                );
+            }
+        }
+        return false;
+    }
+    public function get_aggregate_object_method_params($type, $method)
+    {
+        $definition = $this->_aggr_obj_type[$type];
+        if($definition['methods'][$method]){
+            $method_name = 'api_'.$method.'_'.$type.'_parameters';
+            if(method_exists($this, $method_name)) {
+                $param =  call_user_func_array( array( $this, $method_name), array() );
+                return $param;
+            }
+        }
+        return false;
+    }
+    
+    
     /**
      * @brief
      *           
@@ -758,6 +886,51 @@ class Safe_object_class
         else
             return $type;
     }
+    /**
+     * @brief Return object type base path
+     *
+     * @param[in out] $type
+     *
+     * @return 
+     */
+    public function aggregate_object_base_path($type)
+    {
+        if($this->_aggr_obj_type[$type])
+            return $this->_aggr_obj_type[$type]['base_path'];
+        else
+            return null;
+    }
+    /**
+     * @brief new_fields
+     *
+     * @param[in out] $config_manager
+     * @param[in out] $type
+     * 
+     * Return 
+     *
+     * @return
+     */
+    public function new_fields(&$config_manager, $type)
+    {
+        $obj_def = $this->get_aggregate_object_definition($type);
+        $label = 'New ' . $obj_def['name'];
+        $number = count($this->get_aggregate_objects($type))+1;
+        $new_name = ucfirst($type) . $number;
+        $config_manager->add_field('profile-name', $label, 'string', $new_name, 50);
+        $config_manager->set_field_rules('profile-name', 'required|alpha_dash');
+    }
+
+    /**
+     * @brief Populate auditor with audit point for object
+     *
+     * @param[in out] $auditor
+     *
+     * @return 
+     */
+    public function get_audit_points(&$audit_points)
+    {
+        return true;
+    }
 }
 abstract class Safe_serializable_object_class extends Safe_object_class
 {
@@ -767,6 +940,7 @@ abstract class Safe_serializable_object_class extends Safe_object_class
     private $_status = Safe_object_serializer_class::OBJ_STATUS_NEW;
     private $_id = 0;
     private $_type = Safe_object_serializer_class::OBJ_TYPE_CONFIGURABLE;
+    private $_reuse = true;
     static private $_serializer = null;
     public function __construct($path, $name, $sync_strategy = Safe_serializable_object_class::SYNC_DB)
     {
@@ -777,6 +951,14 @@ abstract class Safe_serializable_object_class extends Safe_object_class
     {
         $this->_sync_strategy = $sync_strategy;
     }
+    /* Set re-use object strategy:
+     * true = keep same record on delete/update just change status
+     * false = change old obj name and set to deleted, then create new one
+     */
+    protected function _set_reuse($reuse=true)
+    {
+        $this->_reuse = $reuse;
+    }
     /**
      * @brief Synchronize serializable object according to startegy
      *         
@@ -818,7 +1000,7 @@ abstract class Safe_serializable_object_class extends Safe_object_class
         $data = $this->_get_serialized_data();
         if (!empty($data)) {
             // Invoke serializer
-            return $this->_serializer()->store_object($this->path() , $this->name() , $this->class_name() , $data, $this->_type, $this->_status, $this->_id);
+            return $this->_serializer()->store_object($this->path() , $this->name() , $this->class_name() , $data, $this->_type, $this->_status, $this->_id, $this->_reuse);
         }
         // No data to serialize
         return FALSE;
@@ -879,6 +1061,11 @@ abstract class Safe_serializable_object_class extends Safe_object_class
     {
         return $this->_serializer()->mark_object_uptodate($this->path() , $this->name());
     }
+    public function mark_modified()
+    {
+        $this->_status = Safe_object_serializer_class::OBJ_STATUS_MODIFIED;
+        return $this->_serializer()->mark_object_modified($this->path() , $this->name());
+    }
     /**
      * @brief Check if the object can be deleted
      *         This function should be implemented in extended objects
@@ -902,23 +1089,13 @@ abstract class Safe_serializable_object_class extends Safe_object_class
      */
     public function dispose()
     {
-        return $this->_serializer()->dispose_object($this->path() , $this->name());
+        return $this->_serializer()->dispose_object($this->path() , $this->name(), $this->_reuse);
     }
     //
     // Return a php array of the data that need to be serialized
     abstract protected function _get_serialized_data();
     // Set the php array of retrieved data
     abstract protected function _set_unserialized_data($data);
-    public function rest_get_data()
-    {
-        $data = array();
-        // Skip our own data if object is configurable
-        if (!is_subclass_of($this, 'Safe_configurable_object_class')) {
-            // Get the serialized data
-            $data = $this->_get_serialized_data();
-        }
-        return array_merge(parent::rest_get_data() , $data);
-    }
     /**
      * @brief get_data_values
      *
@@ -941,6 +1118,9 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
      * @var _group array
      */
     private $_group = array();
+    private $_group_condition = array();
+    private $_default_category = null;
+    private $_default_category_setting = 'expanded';
     /**
      * @brief
      *           
@@ -963,6 +1143,9 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
     {
         return TRUE;
     }
+    public function prepare_edit()
+    {
+    }
     /**
      * @brief Return node object 
      *
@@ -972,31 +1155,24 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
     {
         return $this->_node;
     }
-    /**
-     * @brief Get object data
-     *         
-     * @return array()
-     */
-    public function rest_get_data()
-    {
-        $data = $this->get_data_values_exploded(false, false);
-        $all_data = $data['__all__'];
-        unset($data['__all__']);
-        foreach ($data as $k => $v) $all_data[$k] = $v;
-        $data = parent::rest_get_data();
-        return array_merge($all_data, $data);
-    }
     /**
      * @brief
      *         
      * @return
      */
-    public function enable_disable_enum()
+    public function enable_disable_enum($string_value=true)
     {
-        return array(
-            'true' => 'Enable',
-            'false' => 'Disable'
-        );
+        if($string_value){
+            return array(
+                'true' => 'Enable',
+                'false' => 'Disable'
+            );
+        }else{
+            return array(
+                '1' => 'Enable',
+                '0' => 'Disable'
+            );
+        }
     }
     /**
      * @brief
@@ -1018,6 +1194,10 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
             'default' => $default,
         );
         if (isset($size)) $this->_fields[$name]['size'] = $size;
+        //set defualt category
+        if($this->_default_category != null){
+            $this->set_field_category($name , $this->_default_category, $this->_default_category_setting, false);
+        }
         return TRUE;
     }
     /**
@@ -1100,19 +1280,21 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
         }
         if (strtolower($type) == "checkbox") {
             if (is_array($default) && count($default) > 0) {
-                if (TRUE == $this->add_field($name, $label, $type, $default)) {
+                if (true == $this->add_field($name, $label, $type, $default)) {
                     $this->_fields[$name]['value'] = $enum_array;
                     if($inline) $this->_fields[$name]['inline'] = 'true';
-                    return TRUE;
+                    
                 }
             } elseif (count($default) == 0) {
-                if (TRUE == $this->add_field($name, $label, $type, "")) {
+                if (true == $this->add_field($name, $label, $type, "")) {
                     $this->_fields[$name]['value'] = $enum_array;
                     if($inline) $this->_fields[$name]['inline'] = 'true';
-                    return TRUE;
                 }
             }
+            $this->set_field_rules($name, '');
+            $this->_fields[$name]['rules'] = 'is_array|in_list[' . implode(',', array_keys($this->_fields[$name]['value'])) . ']';
         }
+        return true;
     }
     /**
      * @brief
@@ -1141,9 +1323,15 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
      */
     public function set_field_rules($name, $rules)
     {
-        if ('dropdown' == $this->_fields[$name]['type']) {
+        if ($rules != 'disabled' && in_array($this->_fields[$name]['type'],array("dropdown","radio","checkbox"))) {
             // Ensure enum check is added to rule
-            $rules.= '|required|in_list[' . implode(',', array_keys($this->_fields[$name]['value'])) . ']';
+            if(in_array($this->_fields[$name]['type'],array("dropdown","radio"))){
+                $rules.= '|required';
+            }
+            if(in_array($this->_fields[$name]['type'],array("checkbox"))){
+                $rules.= '|is_array';
+            }
+            $rules.= '|in_list[' . implode(',', array_keys($this->_fields[$name]['value'])) . ']';
         }
         return $this->_set_field_param($name, 'rules', $rules);
     }
@@ -1155,10 +1343,31 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
     {
         return $this->_set_field_param($name, 'readonly', 'readonly');
     }
-    public function set_field_category($name, $category)
+    public function set_field_style($name,$style='')
     {
+        return $this->_set_field_param($name, 'style', $style);
+    }
+    public function set_field_category($name, $category, $setting = 'expanded', $set_as_default = false)
+    {
+        $this->_set_field_param($name, 'category_setting', $setting);
+        if($set_as_default == true){
+            $this->set_default_category($category, $setting);
+        }
         return $this->_set_field_param($name, 'category', $category);
     }
+    public function set_default_category($category, $setting = 'expanded')
+    {
+        $this->_default_category = $category;
+        $this->_default_category_setting = $setting;
+    }
+    public function get_default_category()
+    {
+        return $this->_default_category;
+    }
+    public function get_default_category_setting()
+    {
+        return $this->_default_category_setting;
+    }
     /**
      * @brief
      *           
@@ -1195,6 +1404,14 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
     {
         return $this->_set_field_param($name, 'help', $help);
     }
+    public function get_field_help($name)
+    {
+        $help = '';
+        if (isset($this->_fields[$name]['help'])) {
+            $help = $this->_fields[$name]['help'];
+        }
+        return $help;
+    }
     /**
      * @brief
      *           
@@ -1207,6 +1424,23 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
     {
         return $this->_set_field_param($name, 'events', $events);
     }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $attributes array()
+     *
+     * @return
+     */
+    public function set_field_attributes($name, $attributes = array())
+    {
+        if(is_array($attributes)){
+            foreach($attributes as $key => $value){
+                $this->_set_field_param($name, $key, $value);
+            }
+        }
+        return true;
+    }
     /**
      * @brief
      *           
@@ -1240,13 +1474,58 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
      *           
      * @return
      */
-    public function create_group($groupname, $fields)
+    public function create_group($groupname, $fields , $condition = array('default' => array( 'value' => 'false', 'status' => 'disable')))
     {
         if ($groupname != '' && is_array($fields)) {
             $this->_group[$groupname] = array();
             foreach ($fields as $f) {
-                array_push($this->_group[$groupname], $f);
+                $this->_group[$groupname]['fields'][] = $f;
             }
+            if(isset($condition['default'])){
+                $this->_group[$groupname]['default'] = $condition['default'];
+            }
+            if(isset($condition['condition'])){
+                $this->_group[$groupname]['condition'] = $condition['condition'];
+            }
+            
+        }
+    }
+    public function composite_layout($name, $fields)
+    {
+        if($this->_set_field_param($name, 'composite', $fields)){
+            $_help_msg = array();
+            foreach($fields as $field){
+                $this->_set_field_param($field, 'composite_parent', $name);
+                if (isset($this->_fields[$field]['help'])) {
+                    if(is_array($this->_fields[$field]['help'])){
+                        $field_help =  $this->_fields[$field]['help'];
+                    }else{
+                        $field_help =  array($this->_fields[$field]['help']);
+                    }
+                    $_help_msg = array_merge($_help_msg , array($this->get_data_label($field)), $field_help, array(' '));
+                    unset($this->_fields[$field]['help']);
+                }
+            }
+            if($_help_msg){
+                if(isset($this->_fields[$name]['help'])){
+                    if(is_array($this->_fields[$name]['help'])){
+                       $parent_help =  $this->_fields[$name]['help'];
+                    }else{
+                       $parent_help =  array($this->_fields[$name]['help']);
+                    }
+                    $this->_set_field_param($name, 'help', array_merge(array($this->get_data_label($name)), $parent_help, array(' '), $_help_msg));
+                }else{
+                    $this->_set_field_param($name, 'help',  $_help_msg);
+                }
+            }
+        }
+    }
+
+    public function create_group_condition($field_name, $condition)
+    {
+        if ($field_name != '' && is_array($condition)) {
+            $this->_group_condition[$field_name] = $condition;
+            return $this->_set_field_param($field_name, 'group_condition', 'yes');
         }
     }
     /**
@@ -1258,6 +1537,10 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
     {
         return $this->_group;
     }
+    public function get_group_condition()
+    {
+        return $this->_group_condition;
+    }
     /**
      * @brief Return object data
      *         
@@ -1310,6 +1593,22 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
             return $this->_fields[$name]['value'][$this->_fields[$name]['default']];
         }
     }
+    /**
+     * @brief
+     *
+     * @param[in out] $name
+     * @param[in out] $resolve_enum
+     *
+     * @return
+     */
+    public function get_data_label($name)
+    {
+        if (!isset($this->_fields[$name]['label']) ){
+            return '';
+        }else{
+            return $this->_fields[$name]['label'];
+        }
+    }
     /**
      * @brief Explode data values array into array of data vlues
      *           based on following parameter name pattern
@@ -1327,8 +1626,13 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
         $exploded_values = array();
         foreach ($values as $k => $v) {
             $exploded_key = explode('/', $k);
-            if (2 == sizeof($exploded_key)){
-                $exploded_values[$exploded_key[0]][$exploded_key[1]] = $v;
+            if(sizeof($exploded_key) > 1){
+                // loop around exploded key(s)
+                $entry = &$exploded_values;
+                foreach($exploded_key as $key){
+                    $entry = &$entry[$key];
+                }
+                $entry = $v;
             }else{
                 $exploded_values['__all__'][$k] = $v;
             }
@@ -1374,7 +1678,7 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
         }
         return $values;
     }
-    public function get_data_values_with_field_desc($resolve_enum = true, $include_hidden = true)
+    public function get_data_values_with_field_desc($resolve_enum = true, $include_hidden = true, $include_password = true)
     {
         $values = array();
         // walk data and strip keys, just keep values
@@ -1382,14 +1686,30 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
             if (!$include_hidden && $v['type'] == 'hidden'){
                 continue;
             }
+            if (!$include_password && $v['type'] == 'password'){
+                continue;
+            }
             if (!isset($v['value']) || false == $resolve_enum){
                 $values[$v['label']] = $v['default'];
             }else{
-                if (isset($v['value'][$v['default']])) {
-                    $values[$v['label']] = $v['value'][$v['default']];
-                }else {
-                    $values[$v['label']] = 'INVALID - ' . $v['default'];
-                    //safe_add_error( 'Unable to resolve enum value "'.$v['default'].'" for field '.$k);
+                if(is_array($v['default'])){
+                    foreach($v['default'] as $v1){
+                        if (isset($v['value'][$v1])) {
+                            if($values[$v['label']]){
+                                $values[$v['label']] .= ', ';
+                            }
+                            $values[$v['label']] .= $v['value'][$v1];
+                        }else{
+                            $values[$v['label']] .= 'INVALID - ' . $v1;
+                        }
+                    }
+                }else{
+                    if (isset($v['value'][$v['default']])) {
+                        $values[$v['label']] = $v['value'][$v['default']];
+                    }else {
+                        $values[$v['label']] = 'INVALID - ' . $v['default'];
+                        //safe_add_error( 'Unable to resolve enum value "'.$v['default'].'" for field '.$k);
+                    }
                 }
                 // Adjust known special values
                 if(!$resolve_enum && '__empty__' == $values[$k]){
@@ -1399,6 +1719,22 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
         }
         return $values;
     }
+    public function summary($type = 'horizontal' , $long = false, $min=3)
+    {
+        $list =  $this->get_data_values_with_field_desc(true, false, false);
+        if($long){
+            array_unshift($list,array('Name' => $this->name()));
+            return array( 'data' => $list);
+        }else{
+            $result =  array('Name' => $this->name());
+            foreach($list as $key => $item){
+                $min--;
+                $result[$key] = $item;
+                if($min == 0) break;
+            }
+            return array( 'data' => $result);
+        } 
+    }
     /**
      * @brief
      *           
@@ -1451,7 +1787,7 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
     {
         $values = array();
         // walk data and strip keys, just keep values
-        foreach ($this->_fields as $v) $values[] = $v['label'];
+        foreach ($this->_fields as $v) $values[$v['field']] = $v['label'];
         return $values;
     }
     /**
@@ -1535,12 +1871,15 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
         foreach($data as $key => $value) {
             // Exists
             if($fields[$key]) {
-                if($fields[$key]['type'] == 'hidden' || $fields[$key]['disabled'] == 'disabled')
-                $out_message[$key] = 'Read Only.';
+                if($fields[$key]['type'] == 'hidden' || $fields[$key]['readonly'] == 'readonly') {
+                    unset($data[$key]);//ignore readonly field
+                }
+                if($fields[$key]['disabled'] == 'disabled') {
+                    $this->set_field_rules($key, 'disabled');
+                }
             } else {
                 $out_message[$key] = 'Unknown Parameter.';
             }
-
         }
         // Any error ?
         if($out_message)
@@ -1549,7 +1888,13 @@ class Safe_configurable_object_class extends Safe_serializable_object_class
         $this->set_data_values($data);
         $_POST = $this->get_data_values(false);
         $this->get_form();
+        //set validation obj (for call back check rule)
+        $this->_form->set_validation_obj($this);
         return $this->_form->run_validation($out_message);
     }
+    public function set_form_error($field = '', $message = ''){
+        $this->_form->set_error($field , $message);
+    }
+    
 }
 /* End of file safe_object_class.php */
